/*
  ========================================================================

  FILE:                 AEESQL.h
  
  SERVICES:             ISQL
                        ISQLMgr
                        ISQLStmt
                        ISQLUtil

  GENERAL DESCRIPTION:  Provides database with SQL access

  ========================================================================
  ========================================================================
    
               Copyright  1999-2007 QUALCOMM Incorporated 
                     All Rights Reserved.
                   QUALCOMM Proprietary/GTDR
    
  ========================================================================
  ========================================================================
*/

#ifndef __AEEISQL_H__
#define __AEEISQL_H__

#include "AEE.h"
#include "AEEStdLib.h"
#include "AEEOldVaList.h"

#define AEEIID_SQL         0x0103abca
#define AEEIID_SQLMGR      0x0103abcb
#define AEEIID_SQLUTIL     0x0103abc9
#define AEEIID_SQLSTMT     0x0103abcd

// Forward declarations
typedef struct ISQLMgr     ISQLMgr;
typedef struct ISQL        ISQL;
typedef struct ISQLStmt    ISQLStmt;
typedef struct ISQLUtil    ISQLUtil;

// Special handlers for int64 and doubles
//
// These are used to pass int64 and double quantities in and out 
// of the ISQL object.  This is required because of an inconsistency 
// between how different ARM compilers (ADS vs. RVCT) handle the uint64 
// and double data types.  Whatever you do, don't get all clever and
// access the members of this structure directly.
typedef struct {
   uint32 hi;
   uint32 lo;
} aeesql_uint64_struct;

// retrieve the int64 from an aeesql_uint64_struct
static __inline int64 aeesql_int64struct_to_int64(aeesql_uint64_struct is) {
  union {
     int64 i;
     aeesql_uint64_struct is;
  } iis;

  iis.is = is;
  return iis.i;
}

// construct an aeesql_uint64_struct from an int64
static __inline aeesql_uint64_struct aeesql_int64struct_from_int64(int64 i) {
  union {
     int64 i;
     aeesql_uint64_struct is;
  } iis;

  iis.i = i;
  return iis.is;
}

// retrieve the double from an aeesql_uint64_struct
static __inline double aeesql_doublestruct_to_double(aeesql_uint64_struct ds) {
  union {
     double d;
     aeesql_uint64_struct ds;
  } dds;

  dds.ds = ds;
  return dds.d;
}

// construct an aeesql_uint64_struct from an double
static __inline aeesql_uint64_struct aeesql_doublestruct_from_double(double d) {
  union {
     double d;
     aeesql_uint64_struct ds;
  } dds;

  dds.d = d;
  return dds.ds;
}

//********************************************************************************************************************************
//
// ISQLMgr Interface
//
//********************************************************************************************************************************

// ISQLMgr flags
#define SQLF_WIDE             0x00000001  // Create using wide (16-bit) characters
#define SQLF_JOURNAL          0x00000002  // NOT SUPPORTED.  Journal file is always used.
#define SQLF_NO_FILE_CACHING  0x00000004  // Don't use file caching
#define SQLF_LOAD_EXTENSIONS  0x00000008  // Automatically load extensions

AEEINTERFACE(ISQLMgr)
{
   INHERIT_IQueryInterface(ISQLMgr);

   int    (*Open)(ISQLMgr * po, const char * pszFile, ISQL **piSQL, uint32 flags);
   int    (*Remove)(ISQLMgr * po, const char * pszFile);
};

// IQueryInterface Functions
static __inline uint32 ISQLMGR_AddRef(ISQLMgr *po) {
   return AEEGETPVTBL(po,ISQLMgr)->AddRef(po);
}
static __inline uint32 ISQLMGR_Release(ISQLMgr *po) {
   return AEEGETPVTBL(po,ISQLMgr)->Release(po);
}
static __inline int ISQLMGR_QueryInterface(ISQLMgr *po, AEECLSID idReq, void **ppo) {
   return AEEGETPVTBL(po,ISQLMgr)->QueryInterface(po, idReq, ppo);
}

// SQL DB Manager Functions
static __inline int ISQLMGR_Open(ISQLMgr *po, const char *pszFile, ISQL **piSQL, uint32 flags) {
   return AEEGETPVTBL(po,ISQLMgr)->Open(po, pszFile, piSQL, flags);
}
static __inline int ISQLMGR_Remove(ISQLMgr *po, const char *pszFile) {
   return AEEGETPVTBL(po,ISQLMgr)->Remove(po, pszFile);
}

//********************************************************************************************************************************
//
// ISQLStmt Interface
//
//********************************************************************************************************************************

// Data types
typedef enum {
   AEESQL_DT_TEXT,
   AEESQL_DT_WTEXT,
   AEESQL_DT_INT,
   AEESQL_DT_DOUBLE,
   AEESQL_DT_BLOB,
   AEESQL_DT_NULL,
   AEESQL_DT_INT64
} AEESQLDataType;

typedef enum {
   AEESQL_COL_DT_INTEGER = 1,
   AEESQL_COL_DT_FLOAT,
   AEESQL_COL_DT_TEXT,
   AEESQL_COL_DT_BLOB,
   AEESQL_COL_DT_NULL
} AEESQLColDataType;

// Callback function to free bind data
typedef void (*SQLFreeCallBack)(void* pData);
#define SQL_STATIC      ((void(*)(void *))0)
#define SQL_TRANSIENT   ((void(*)(void *))-1)

AEEINTERFACE(ISQLStmt)
{
   INHERIT_IQueryInterface(ISQLStmt);

   int   (*Step)(ISQLStmt *piStmt);
   int   (*Reset)(ISQLStmt *piStmt);
   int   (*GetColumn)(ISQLStmt *piStmt, AEESQLDataType type, int column, void *pData, int size);
   int   (*ColumnCount)(ISQLStmt *piStmt);
   int   (*ColumnSize)(ISQLStmt *piStmt, int column);
   AEESQLColDataType (*ColumnDataType)(ISQLStmt *piStmt, int column);
   const void * (*ColumnDeclType)(ISQLStmt *piStmt, int column, uint32 flags);
   const void * (*ColumnName)(ISQLStmt *piStmt, int column, uint32 flags);
   int   (*Bind)(ISQLStmt *piStmt, AEESQLDataType type, int column, void *pData, int nSize, SQLFreeCallBack pFree);
   int   (*BindCount)(ISQLStmt *piStmt);
   int   (*BindIndex)(ISQLStmt *piStmt, const char *pName);
   const char* (*BindName)(ISQLStmt *piStmt, int index);
};

// IQueryInterface Functions
static __inline uint32 ISQLSTMT_AddRef(ISQLStmt *piStmt) {
   return AEEGETPVTBL(piStmt,ISQLStmt)->AddRef(piStmt);
}
static __inline uint32 ISQLSTMT_Release(ISQLStmt *piStmt) {
   return AEEGETPVTBL(piStmt,ISQLStmt)->Release(piStmt);
}
static __inline int ISQLSTMT_QueryInterface(ISQLStmt *piStmt, AEECLSID idReq, void **ppiStmt) {
   return AEEGETPVTBL(piStmt,ISQLStmt)->QueryInterface(piStmt, idReq, ppiStmt);
}

// Statement execution
static __inline int ISQLSTMT_Step(ISQLStmt *piStmt) {
   return AEEGETPVTBL(piStmt,ISQLStmt)->Step(piStmt);
}
static __inline int ISQLSTMT_Reset(ISQLStmt *piStmt) {
   return AEEGETPVTBL(piStmt,ISQLStmt)->Reset(piStmt);
}

// Get column data
static __inline int ISQLSTMT_GetColumn(ISQLStmt *piStmt, AEESQLDataType type, int column, void *pData, int size) {
   return AEEGETPVTBL(piStmt,ISQLStmt)->GetColumn(piStmt, type, column, pData, size);
}
static __inline int ISQLSTMT_ColumnCount(ISQLStmt *piStmt) {
   return AEEGETPVTBL(piStmt,ISQLStmt)->ColumnCount(piStmt);
}
static __inline int ISQLSTMT_ColumnSize(ISQLStmt *piStmt, int column) {
   return AEEGETPVTBL(piStmt,ISQLStmt)->ColumnSize(piStmt, column);
}
static __inline AEESQLColDataType ISQLSTMT_ColumnDataType(ISQLStmt *piStmt, int column) {
   return AEEGETPVTBL(piStmt,ISQLStmt)->ColumnDataType(piStmt, column);
}
static __inline const void * ISQLSTMT_ColumnDeclTypeEx(ISQLStmt *piStmt, int column, uint32 flags) {
   return AEEGETPVTBL(piStmt,ISQLStmt)->ColumnDeclType(piStmt, column, flags);
}
static __inline const void * ISQLSTMT_ColumnNameEx(ISQLStmt *piStmt, int column, uint32 flags) {
   return AEEGETPVTBL(piStmt,ISQLStmt)->ColumnName(piStmt, column, flags);
}

// Column data convenience functions
static __inline const char * ISQLSTMT_ColumnName(ISQLStmt *piStmt, int column) {
   return (const char *)ISQLSTMT_ColumnNameEx(piStmt, column, 0);
}
static __inline const AECHAR * ISQLSTMT_WColumnName(ISQLStmt *piStmt, int column) {
   return (const AECHAR *)ISQLSTMT_ColumnNameEx(piStmt, column, SQLF_WIDE);
}
static __inline const char * ISQLSTMT_ColumnDeclType(ISQLStmt *piStmt, int column) {
   return (const char *)ISQLSTMT_ColumnDeclTypeEx(piStmt, column, 0);
}
static __inline const AECHAR * ISQLSTMT_WColumnDeclType(ISQLStmt *piStmt, int column) {
   return (const AECHAR *)ISQLSTMT_ColumnDeclTypeEx(piStmt, column, SQLF_WIDE);
}
static __inline int ISQLSTMT_GetColumnBlob(ISQLStmt *piStmt, int column, void *pBlob) {
   return ISQLSTMT_GetColumn(piStmt, AEESQL_DT_BLOB, column, pBlob, sizeof(void*));
}
static __inline int ISQLSTMT_GetColumnInt(ISQLStmt *piStmt, int column, int *pInt) {
   return ISQLSTMT_GetColumn(piStmt, AEESQL_DT_INT, column, pInt, sizeof(int));
}
static __inline int ISQLSTMT_GetColumnInt64(ISQLStmt *piStmt, int column, int64 *pInt64) {
   aeesql_uint64_struct stResult;
   int nResult = ISQLSTMT_GetColumn(piStmt, AEESQL_DT_INT64, column, &stResult, sizeof(stResult));
   *pInt64 = aeesql_int64struct_to_int64(stResult);
   return nResult;
}
static __inline int ISQLSTMT_GetColumnDouble(ISQLStmt *piStmt, int column, double *pDouble) {
   aeesql_uint64_struct stResult;
   int nResult = ISQLSTMT_GetColumn(piStmt, AEESQL_DT_DOUBLE, column, &stResult, sizeof(stResult));
   *pDouble = aeesql_doublestruct_to_double(stResult);
   return nResult;
}
static __inline int ISQLSTMT_GetColumnText(ISQLStmt *piStmt, int column, char **pText) {
   return ISQLSTMT_GetColumn(piStmt, AEESQL_DT_TEXT, column, pText, sizeof(char*));
}
static __inline int ISQLSTMT_GetColumnWText(ISQLStmt *piStmt, int column, AECHAR **pWText) {
   return ISQLSTMT_GetColumn(piStmt, AEESQL_DT_WTEXT, column, pWText, sizeof(AECHAR*));
}

// Bind data to query
static __inline int ISQLSTMT_BindData(ISQLStmt *piStmt, AEESQLDataType type, int column, void *pData, int nSize, SQLFreeCallBack pFree) {
   return AEEGETPVTBL(piStmt,ISQLStmt)->Bind(piStmt, type, column, pData, nSize, pFree);
}
static __inline int ISQLSTMT_BindCount(ISQLStmt *piStmt) {
   return AEEGETPVTBL(piStmt,ISQLStmt)->BindCount(piStmt);
}
static __inline int ISQLSTMT_BindIndex(ISQLStmt *piStmt, const char *pName) {
   return AEEGETPVTBL(piStmt,ISQLStmt)->BindIndex(piStmt, pName);
}
static __inline const char* ISQLSTMT_BindName(ISQLStmt *piStmt, int index) {
   return AEEGETPVTBL(piStmt,ISQLStmt)->BindName(piStmt, index);
}

// Bind convenience functions
static __inline int ISQLSTMT_BindBlob(ISQLStmt *piStmt, int wildcard, void *pBlob, int nSize, SQLFreeCallBack pCB) {
   return ISQLSTMT_BindData(piStmt, AEESQL_DT_BLOB, wildcard, pBlob, nSize, pCB);
}
static __inline int ISQLSTMT_BindInt(ISQLStmt *piStmt, int wildcard, int value) {
   return ISQLSTMT_BindData(piStmt, AEESQL_DT_INT, wildcard, &value, sizeof(int), NULL);
}
static __inline int ISQLSTMT_BindInt64(ISQLStmt *piStmt, int wildcard, int64 value) {       
   aeesql_uint64_struct stTmp = aeesql_int64struct_from_int64(value);
   return ISQLSTMT_BindData(piStmt, AEESQL_DT_INT64, wildcard, &stTmp, sizeof(aeesql_uint64_struct), NULL); 
}
static __inline int ISQLSTMT_BindDouble(ISQLStmt *piStmt, int wildcard, double value) {
   aeesql_uint64_struct stTmp = aeesql_doublestruct_from_double(value);
   return ISQLSTMT_BindData(piStmt, AEESQL_DT_DOUBLE, wildcard, &stTmp, sizeof(aeesql_uint64_struct), NULL);
}
static __inline int ISQLSTMT_BindText(ISQLStmt *piStmt, int wildcard, char *pText, int nSize, SQLFreeCallBack pCB) {
   return ISQLSTMT_BindData(piStmt, AEESQL_DT_TEXT, wildcard, pText, nSize, pCB);
}
static __inline int ISQLSTMT_BindWText(ISQLStmt *piStmt, int wildcard, AECHAR *pWText, int nSize, SQLFreeCallBack pCB) {
   return ISQLSTMT_BindData(piStmt, AEESQL_DT_WTEXT, wildcard, pWText, nSize, pCB);
}
static __inline int ISQLSTMT_BindNull(ISQLStmt *piStmt, int wildcard) {
   return ISQLSTMT_BindData(piStmt, AEESQL_DT_NULL, wildcard, NULL, 0, NULL);
}

//********************************************************************************************************************************
//
// ISQL Interface
//
//********************************************************************************************************************************

// Callback function for ISQL_Exec
typedef int (*SQLExecCallBack)(void* pUserData,int nCols,char** ppColValues, char** ppColNames);

typedef enum {
   SGI_ROWS_CHANGED = 1,   // rows change (or inserted or deleted) by most recent INSERT, UPDATE, or DELETE statement
   SGI_TOTAL_CHANGED,      // total rows modified, inserted, or deleted since database was opened
   SGI_LAST_ROWID,         // integer key of most recent insert
   SGI_ERROR_MSG           // Error message for the most recent failed ISQL call
} AEESQLGetInfoType;

// ISQL Interface
AEEINTERFACE(ISQL)
{
   INHERIT_IQueryInterface(ISQL);

   int   (*Exec)(ISQL *piSQL, const char *pSQL, SQLExecCallBack pCallback, void* pUserData, char** ppErrorMsg);
   int   (*Prepare)(ISQL *piSQL, const void *pSQL, int len, uint32 flags, ISQLStmt **ppStmt, const void **ppsTail);
   int   (*GetError)(ISQL *piSQL);
   int   (*GetInfo)(ISQL *piSQL, AEESQLGetInfoType type, void *pData, int *pSize);
};

// IQueryInterface Functions
static __inline uint32 ISQL_AddRef(ISQL *po) {
   return AEEGETPVTBL(po,ISQL)->AddRef(po);
}
static __inline uint32 ISQL_Release(ISQL *po) {
   return AEEGETPVTBL(po,ISQL)->Release(po);
}
static __inline int ISQL_QueryInterface(ISQL *po, AEECLSID idReq, void **ppo) {
   return AEEGETPVTBL(po,ISQL)->QueryInterface(po, idReq, ppo);
}

// Simple query
static __inline int ISQL_Exec(ISQL *po, const char *pSQL, SQLExecCallBack pCallback, void* pUserData, char** ppErrorMsg) {
   return AEEGETPVTBL(po,ISQL)->Exec(po, pSQL, pCallback, pUserData, ppErrorMsg);
}

// Prepare query statement
static __inline int ISQL_PrepareEx(ISQL *po, const void *pSQL, int len, uint32 flags, ISQLStmt **ppStmt, const void **ppsTail) {
   return AEEGETPVTBL(po,ISQL)->Prepare(po, pSQL, len, flags, ppStmt, ppsTail);
}
static __inline int ISQL_Prepare(ISQL *piSQL, const char *pSQL, int len, ISQLStmt **ppStmt, const char **ppsTail) {
   return ISQL_PrepareEx(piSQL, pSQL, len, 0, ppStmt, (const void **)ppsTail);
}
static __inline int ISQL_WPrepare(ISQL *piSQL, const AECHAR *pSQL, int len, ISQLStmt **ppStmt, const AECHAR **ppsTail) {
   return ISQL_PrepareEx(piSQL, pSQL, len, SQLF_WIDE, ppStmt, (const void **)ppsTail);
}

// Miscellaneous
static __inline int ISQL_GetError(ISQL *po) {
   return AEEGETPVTBL(po,ISQL)->GetError(po);
}
static __inline int ISQL_GetInfo(ISQL *po, AEESQLGetInfoType type, void *pData, int *pSize) {
   return AEEGETPVTBL(po,ISQL)->GetInfo(po, type, pData, pSize);
}

// ISQL_GetInfo convenience functions
static __inline int ISQL_GetRowsChanged(ISQL *piSQL) {
   int   result;
   int   dwSize = sizeof(result);
   (void)ISQL_GetInfo(piSQL, SGI_ROWS_CHANGED, &result, &dwSize); 
   return result;
}

static __inline int ISQL_GetTotalChanged(ISQL *piSQL) {
   int   result;
   int   dwSize = sizeof(result);
   (void)ISQL_GetInfo(piSQL, SGI_TOTAL_CHANGED, &result, &dwSize); 
   return result;
}

static __inline int64 ISQL_GetLastRowId(ISQL *piSQL) {
   aeesql_uint64_struct stResult;
   int dwSize = sizeof(stResult);
   (void)ISQL_GetInfo(piSQL, SGI_LAST_ROWID, &stResult, &dwSize);
   return aeesql_int64struct_to_int64(stResult);
}

static __inline int ISQL_GetErrorMsgSize(ISQL *piSQL) {
   int  msgSize = 0;
   (void) ISQL_GetInfo(piSQL, SGI_ERROR_MSG, NULL, &msgSize); // NULL pData == size request
   return msgSize;
}

static __inline int ISQL_GetErrorMsg(ISQL *piSQL, char *msg, int size) {
   return ISQL_GetInfo(piSQL, SGI_ERROR_MSG, msg, &size);
}

//********************************************************************************************************************************
//
// ISQLUtil Interface
//
//********************************************************************************************************************************

AEEINTERFACE(ISQLUtil)
{
   INHERIT_IQueryInterface(ISQLUtil);

   int   (*vxnprintf)(ISQLUtil *piUtil, int useInternal, char *buf, uint32 f, const char *format, AEEOldVaList list);
};

// IQueryInterface Functions
static __inline uint32 ISQLUTIL_AddRef(ISQLUtil *piUtil) {
   return AEEGETPVTBL(piUtil,ISQLUtil)->AddRef(piUtil);
}
static __inline uint32 ISQLUTIL_Release(ISQLUtil *piUtil) {
   return AEEGETPVTBL(piUtil,ISQLUtil)->Release(piUtil);
}
static __inline int ISQLUTIL_QueryInterface(ISQLUtil *piUtil, AEECLSID idReq, void **ppiUtil) {
   return AEEGETPVTBL(piUtil,ISQLUtil)->QueryInterface(piUtil, idReq, ppiUtil);
}

// vxprintf
static __inline int ISQLUTIL_vxnprintf(ISQLUtil *piUtil, int useInternal, char *buf, uint32 f, const char *format, va_list args) {
   return AEEGETPVTBL(piUtil,ISQLUtil)->vxnprintf(piUtil, useInternal, buf, f, format, AEEOldVaList_From_va_list((va_list *)(&args)));
}

//********************************************************************************************************************************
//
// Error Codes
//
//********************************************************************************************************************************
#define SQL_ERRBASE           (0x7000)
#define SQL_OK                SUCCESS           /* Successful result */
#define SQL_ERROR             (SQL_ERRBASE+1)   /* ISQL error or missing database */
#define SQL_INTERNAL          (SQL_ERRBASE+2)   /* An internal logic error in SQLite */
#define SQL_PERM              (SQL_ERRBASE+3)   /* Access permission denied */
#define SQL_ABORT             (SQL_ERRBASE+4)   /* Callback routine requested an abort */
#define SQL_BUSY              (SQL_ERRBASE+5)   /* The database file is locked */
#define SQL_LOCKED            (SQL_ERRBASE+6)   /* A table in the database is locked */
#define SQL_NOMEM             (SQL_ERRBASE+7)   /* A malloc() failed */
#define SQL_READONLY          (SQL_ERRBASE+8)   /* Attempt to write a readonly database */
#define SQL_INTERRUPT         (SQL_ERRBASE+9)   /* Operation terminated by SQL_interrupt() */
#define SQL_IOERR             (SQL_ERRBASE+10)  /* Some kind of disk I/O error occurred */
#define SQL_CORRUPT           (SQL_ERRBASE+11)  /* The database disk image is malformed */
#define SQL_NOTFOUND          (SQL_ERRBASE+12)  /* (Internal Only) Table or record not found */
#define SQL_FULL              (SQL_ERRBASE+13)  /* Insertion failed because database is full */
#define SQL_CANTOPEN          (SQL_ERRBASE+14)  /* Unable to open the database file */
#define SQL_PROTOCOL          (SQL_ERRBASE+15)  /* Database lock protocol error */
#define SQL_EMPTY             (SQL_ERRBASE+16)  /* (Internal Only) Database table is empty */
#define SQL_SCHEMA            (SQL_ERRBASE+17)  /* The database schema changed */
#define SQL_TOOBIG            (SQL_ERRBASE+18)  /* Too much data for one row of a table */
#define SQL_CONSTRAINT        (SQL_ERRBASE+19)  /* Abort due to constraint violation */
#define SQL_MISMATCH          (SQL_ERRBASE+20)  /* Data type mismatch */
#define SQL_MISUSE            (SQL_ERRBASE+21)  /* Library used incorrectly */
#define SQL_NOLFS             (SQL_ERRBASE+22)  /* Uses OS features not supported on host */
#define SQL_AUTH              (SQL_ERRBASE+23)  /* Authorization denied */
#define SQL_ROW               (SQL_ERRBASE+100) /* SQLStmt_Step() has another row ready */
#define SQL_DONE              (SQL_ERRBASE+101) /* SQLStmt_Step() has finished executing */
#define SQL_AUTOLOADLIBERROR  (SQL_ERRBASE+1000)/* Database was opened, but an error occurred auto-loading libraries */

/////////////////////////////////////////////////////////////////////////////

/*==============================================================================
  DATA STRUCTURE DOCUMENTATION
================================================================================
ISQLMgr Interface and Class IDs

Description:
   Each class and interface is identified by a unique ID.  These IDs are used to 
   distinguish between various types of objects.

   ISQLMgr supports both an interface and the creation of class object instances.  
   The ISQLMgr interface or a new instance of the ISQLMgr class object may be 
   created by calling ISHELL_CreateInstance() and passing in the appropriate 
   class id.

   ISQLMgr recognizes the following set of class and interface IDs.

Definition:

Identifier           Definition
----------           ---------------------------------------------------------
AEEIID_SQLMGR:       Identifies the ISQLMgr interface
AEECLSID_SQLMGR:     Identifies the ISQLMgr implementation in the SQL extension

Comments:
   None

See Also:
   ISHELL_CreateInstance()

================================================================================

ISQLMgr Flags

Description:
   The ISQLMgr is used to manage and open databases.  The ISQLMGR_Open() 
   routine is used to create a new database file or open a new connection to
   an existing database file.  This function's behavior is governed by a set
   of flags passed to it, defined here:
   
Definition:
===pre>
   Definition              Description
   ----------------------  -----------------------------------------------------
   SQLF_WIDE:              This flag instructs ISQLMGR_Open() to create the
                           database using wide (16-bit) characters.

   SQLF_NO_FILE_CACHING:   Tells the ISQL object not to use BREW's file caching
                           mechanism for a possible performace gain.

   SQLF_LOAD_EXTENSIONS:   When this flag is set, ISQLMGR_Open() reads the list 
                           of user-defined function libraries listed in a 
                           special table in the database file and automatically 
                           loads them.  If a library is found in in the auto-
                           load table that's not discoverable via a call to
                           ISHELL_QueryClass(), it will be permanently removed.
===/pre>

Members:
   None
   
Comments:
   None

See Also:
   ISQLMGR_Open()

================================================================================
 INTERFACE DOCUMENTATION
================================================================================

ISQLMgr Interface

Description: 
   The ISQLMgr interface functions are used to create, open and remove 
   databases. Once a database has been opened, you use functions in the ISQL 
   Interface to add, update and query database information.

   Your application must have a privilege level of File or All to be able to 
   invoke the functions in this interface that create or delete the database.

   The function ISQLMGR_Open() opens a database given the name of a BREW file 
   that holds the database's contents. If the file doesn't already exist, it 
   will be created. This function returns a pointer to an instance of the ISQL 
   interface, which can be used to access the opened database.

   The function ISQLMGR_Remove() removes a database by deleting the file that 
   holds its contents. If the database is open, you should first call 
   ISQL_Release() to close it prior to removal.

   To use the functions in the ISQLMgr interface call ISHELL_CreateInstance() 
   to create an instance of the ISQLMgr interface. Call ISQLMGR_Open() to open 
   an existing database or to create a new one.  Using the ISQL interface 
   pointer, call functions in the ISQL interface to create, update and retrieve  
   database records. Call ISQL_Release() to close the database when you have 
   completed accessing it.  Call ISQLMGR_Remove() to remove the database when 
   necessary. Call ISQLMGR_Release() when you no longer need the ISQLMgr 
   Interface instance.

===H2>
Required header files
===/H2>
===pre>
AEESQL.h
===/pre>

See Also:
   ISQL

================================================================================

ISQLMGR_AddRef() 

Description: 
   This function increments the reference count of the ISQLMgr Interface object.  
   This allows the object to be shared by multiple callers. The object is freed 
   when the reference count reaches 0 (zero).

Prototype:
   uint32 ISQLMGR_AddRef(ISQLMgr *pISQLMgr); 

Parameters:
===pre>
   pISQLMgr: Pointer to the ISQLMgr Interface object
===/pre> 

Return Value:
   Incremented reference count for the object. 

Comments: 
   A valid object returns a positive reference count.

See Also:
   ISQLMGR_Release() 

================================================================================ 

ISQLMGR_Open() 

Description:
   This function opens the specified database. A database may be opened more 
   than once simultaneously by multiple applications. If  successful, an 
instance of ISQL interface is returned.

Prototype:
   int ISQLMGR_Open(ISQLMgr * po, 
                    const char * pszFile, 
                    ISQL **piSQL, 
                    uint32 flags);

Parameters:
===pre>
   po:      Pointer to the ISQLMgr Interface object
   pszFile: Null terminated string denoting the database file name
   piSQL:   Address of variable to hold ISQL pointer result
   flags:   The following flags may be used:
            SQLF_WIDE - Create database using wide (16-bit) characters         
            SQLF_NO_FILE_CACHING - Don't use file caching. Setting this flag
             may increase performance for some operations.
            SQLF_LOAD_EXTENSIONS - load the libraries listed in the table of
             user-defined function extensions, and clean the auto-load table
             of libraries that aren't present on the system.
===/pre>

Return Value:
===pre>
   SUCCESS if the database was successfully opened
   SQL_AUTOLOADLIBERROR if SQLF_LOAD_EXTENSIONS was specified but the libraries
    could not be opened.  piSQL pointer will be valid in this case.
   Error otherwise
===/pre>

Comments
   Use ISQL_Release() to close the database.

================================================================================

ISQLMGR_Remove() 

Description: 
   This function removes the specified database.

Prototype:
   int ISQLMGR_Remove(ISQLMgr * po, const char * pszFile); 

Parameters:
===pre>
   po:        ISQLMgr * pointer to ISQLMgr object
   pszFile:   Null terminated string specifying the name of the database 
              to be removed
===/pre>

Return Value: 
   SUCCESS if the database was successfully removed

Comments: 
   Once the database is removed, the corresponding database file is deleted. 
  
================================================================================

ISQLMGR_QueryInterface()

Description:
   This function queries an interface for an alterrnate interface that may 
   provide extended functionality or a proprietary extension.  If successful, 
   the function will return a pointer to the requested interface in the memory 
   location pointed to by the 'd' parameter.  The requested interface is 
   identified by a globally unique interface ID passed to the function in the 
   'c' parameter.

Prototype:
   int ISQLMGR_QueryInterface(ISQLMgr *p, AEECLSID c, void **d);

Parameters:
===pre>
   p:  Pointer to the ISQLMgr interface object.
   c:  A globally unique id that identifies the interface to be queried.
   d:  Pointer to a pointer returned by the function to reference the
       interface requested by the 'c' parameter.  If the queried item is not
       available, this pointer is returned as NULL.
===/pre> 

Return Value:
===pre>
   SUCCESS          - The requested interface was found and has been returned
                      in the 'd' parameter.
   ECLASSNOTSUPPORT - The requested interface is not supported.
===/pre> 

Comments:
   The pointer passed in the 'd' parameter is a double-indirection -- i.e. it 
   is a pointer to a pointer.  Applications should use care in ensuring that the 
   proper level of indirection is used in calling the function.

Side Effects:
   Upon successful retrieval of an interface pointer, that interface is returned 
   with its reference count incremented.  The caller is responsible for 
   releasing this object at some point in the future.

See Also:
   IQI_QueryInterface()

================================================================================

ISQLMGR_Release() 

Description: 
   This function decrements the reference count of the ISQLMgr Interface object. 
   The object is freed from memory and is no longer valid once the reference 
   count reaches 0 (zero). 

Prototype:
   uint32 ISQLMGR_Release(ISQLMgr *pISQLMgr);
    
Parameters:
===pre>
   pISQLMgr: Pointer to the ISQLMgr Interface object
===/pre> 

Return Value:
===pre> 
   The decremented reference count for the object.
   0 (zero) if the object has been freed and is no longer valid.
===/pre> 

See Also:
   ISQLMGR_AddRef() 

================================================================================
  DATA STRUCTURE DOCUMENTATION
================================================================================
ISQL Interface and Class IDs

Description:
   Each class and interface is identified by a unique ID.  These IDs are used to 
   distinguish between various types of objects.

   ISQL supports both an interface and the creation of class object instances.  
   The ISQL interface or a new instance of the ISQL class object may be created 
   by calling ISHELL_CreateInstance() and passing in the appropriate class id.

   ISQL recognizes the following set of class and interface IDs.

Definition:

Identifier           Definition
----------           ---------------------------------------------------------
AEEIID_SQL:          Identifies the ISQL interface
AEECLSID_SQL:        Identifies the ISQL implementation in the SQL extension

Comments:
   None

See Also:
   ISHELL_CreateInstance()

================================================================================

ISQL Error Codes

Description:
   The ISQL interfaces may return the following error codes:
   
Definition:
===pre>
   Definition              Description
   ----------------------  ----------------------------------------------------
   SQL_OK                  Successful result
   SQL_ERROR               ISQL error or missing database
   SQL_INTERNAL            An internal logic error in SQLite
   SQL_PERM                Access permission denied
   SQL_ABORT               Callback routine requested an abort
   SQL_BUSY                The database file is locked
   SQL_LOCKED              A table in the database is locked
   SQL_NOMEM               A malloc() failed
   SQL_READONLY            Attempt to write a readonly database
   SQL_INTERRUPT           Operation terminated by SQL_interrupt()
   SQL_IOERR               Some kind of disk I/O error occurred
   SQL_CORRUPT             The database disk image is malformed
   SQL_NOTFOUND            (Internal Only) Table or record not found
   SQL_FULL                Insertion failed because database is full
   SQL_CANTOPEN            Unable to open the database file
   SQL_PROTOCOL            Database lock protocol error
   SQL_EMPTY               (Internal Only) Database table is empty
   SQL_SCHEMA              The database schema changed
   SQL_TOOBIG              Too much data for one row of a table
   SQL_CONSTRAINT          Abort due to constraint violation
   SQL_MISMATCH            Data type mismatch
   SQL_MISUSE              Library used incorrectly
   SQL_NOLFS               Uses OS features not supported on host
   SQL_AUTH                Authorization denied
   SQL_ROW                 SQLStmt_Step() has another row ready
   SQL_DONE                SQLStmt_Step() has finished executing
   SQL_AUTOLOADLIBERROR    The database was opened, but an error occured 
                           loading the libraries listed in the ato-load table
===/pre>

Members:
   None
   
Comments:
   None

See Also:
   ISQL

================================================================================

AEESQLGetInfoType

Description:
   The ISQL_GetInfo() call can be used to retrieve different information from
   ISQL.  The constants defined below can be passed to this routine to
   retrieve different bits of information.  Applications may also use the
   ISQL_GetRowsChanged(), ISQL_GetTotalChanged(), ISQL_GetLastRowId() or
   ISQL_GetErrorMsg() inlines as shortcuts.
   
Definition:
===pre>
   Definition              Description
   ----------------------  -----------------------------------------------------
   SGI_ROWS_CHANGED        Retreive the number of rows changed (or inserted or 
                           deleted) by most recent INSERT, UPDATE, or DELETE 
                           statement.

   SGI_TOTAL_CHANGED       Retrieve the total number of rows modified, inserted 
                           or deleted since the database was opened.

   SGI_LAST_ROWID          Retrieve the integer key of the most recent insert, 
                           an int64 value.

   SGI_ERROR_MSG           Retrieve the error message for the most recent 
                           failed ISQL call.
===/pre>

Members:
   None
   
Comments:
   None

See Also:
   ISQL_GetInfo()
   ISQL_GetRowsChanged() 
   ISQL_GetTotalChanged()
   ISQL_GetLastRowId()
   ISQL_GetErrorMsg()

================================================================================
 INTERFACE DOCUMENTATION
================================================================================

ISQL Interface

Description: 
   The ISQL interface functions allow you to execute SQL statements on an open 
   database. The simplest approach is to call ISQL_Exec passing in the text of 
   the SQL statements to execute and a pointer to a callback function that will 
   receive any query results. Results are returned a row at a time with the 
   value of each column in text format.

   An alternative approach is to use ISQL_Prepare to compile the SQL text. This 
   returns the compiled statements in the form of an ISQLStmt interface from 
   which you can step through the SQL statements and get the results in various 
   formats. This allows you to asynchronously process SQL and receive results. 
   This approach also allows you to easily add or update records with non-text 
   data such integers, floating point numbers, and binary blobs by binding data 
   to specified columns.

===H2>
Performance and Resource Usage
===/H2>
   ISQL's internal SQL implementation (SQLite) is designed to provide 
   the most durable embedded database system possible.  As such, it
   uses a journal file to maintain database integrity across
   unexpected device powerdowns and application crashes.  The journal
   file is used to restore the database file to its previous state 
   after an unexpected exit.

   While the journal file provides database durability, it introduces
   several caveats.  The journal file is used to store pages from the
   database before an operation is committed.  This requires both the
   existing data in the database to be written to the file system as 
   well as the new data being comitted.  These extra file system calls
   decrease ISQL's overal performace.  Database designers should keep
   this in mind and store only what's necessay with their databases.  
   For example, a content management database ought to be used to 
   store DRM information, but not the actual protected content itself.

   The use of the journal file also increases the file system resources
   ISQL requires to perform transactions.  If the file system space 
   is not available for the journal file, the SQL transaction will 
   fail.  For example, a transaction to delete an item from a 
   database requires enough file system space for a journal file 
   containing the original item.  If the file system is full, ISQL 
   will be unable to create this journal file and the operation will
   file.  ISQL will be unable to remove items from its database file.  
   See the  section on configuring ISQL below to tell ISQL to recycle 
   its journal file and essentially "reserve" file system space to 
   avoid this potential problem.

===H2>
Configuring ISQL
===/H2>
   ISQL may be configured through it's BREW APIs or by executing
   "pragma" commands through ISQL_Exec().  The underlying SQL 
   implementation recognizes this command and can be configured
   at runtime in various ways.  The configuration established by
   the use of the pragma command is typically valid only for the
   lifetime of the ISQL object.  Next time you open the database, 
   the same pragma commands should be executed.  

   For example, to tell ISQL to reserve a particular amount of 
   file system space (by reusing its journal file as described in 
   the previous section), an application might execute the following 
   SQL query:

===pre>
      PRAGMA BREW_reserved_fs_bytes=5000
===/pre>

   ISQL will interpret this query as a configuration command, and
   perform the necessary internal tweaks to recycle its journal
   file, keeping it at 5k to reserve file system space for the next
   SQL operation.

   A subset of the pragma commands ISQL understands follows:

===H3>
   File System Space Reservation
===/H3>
   The BREW_reserved_fs_bytes pragma tells ISQL to reserve the specified
   numnber of bytes in the file system by resizing the journal file to
   the requested size and saving it for a future operation.  If there's
   a chance your application might fill up EFS space, it is recommended
   to use this pragma and reserve enough space to delete a single item
   from  your database.

   To reserve 5k in EFS for ISQL operations, execute the following
   pragma command:
===pre>
      PRAGMA BREW_reserved_fs_bytes=5000;
===/pre>
   Note that once this pragma has been set, the journal file will be
   recycled and reused for each and every transaction.  The performance
   impact of this command is negligible.

===H3>
   Connection Cache Size
===/H3>
   The cache size pragmas are used to control the number of database 
   pages a session can hold in memory.  This value affects how many
   operations can be cached befome the SQL implementation requires an
   exclusive lock on the database to commit them.  If your application
   accesses a database that is simultaneously shared by other ISQL
   instances, increasing the cache size may reduce the occurences
   of SQL_BUSY errors at the expense of extra memory usage.

   To set the cache size for a particular session, excecute the
   following pragma command:
===pre>
      PRAGMA cache_size=10000;
===/pre>
   To permanently set the cache size for all sessions, the following
   pragma command should be executed:
===pre>
      PRAGMA default_cache_size=10000;
===/pre>
   The default cache_size value used by ISQL is 2000 pages. 

===H3>
   Getting Database Information
===/H3>
   The following pragmas can be used to obtain information about 
   the current database:

   The database_list pragma provides a list of all attached databases.
===pre>
      PRAGMA database_list;
===/pre>
   The index_info pragma lists information about columns with an 
   index in a particular table.  This pragma takes an index name as 
   an argument.
===pre>
     CREATE INDEX my_index ON mytable(name, type_id);
     PRAGMA index_info(my_index);
===/pre>
   The index_list pragma provides information about all indexes in a 
   table.  This pragma takes a table name as an argument.
===pre>
      PRAGMA index_list(mytable);
===/pre>
   The table_info pragma provides information about all columns in a 
   table.  This pragma takes a table name as an argument.
===pre>
   PRAGMA table_info(mytable);
===/pre>

===H3>
   Debugging
===/H3>
   The integrity_check pragma can be used to verify the integrity of
   a database file.  When executed, ISQL looks for out-of-order
   records, missing pages and other corruption.  A single string is
   returned listing the problems if any are found.
===pre>
      PRAGMA integrity_check;
===/pre>

   For a complete list of pragma commands ISQL understands, visit
===pre>
      http://sqlite.org/pragma.html
===/pre>
   Note that some commands listed there may not be applicable to the 
   SQLite implementation in this version of ISQL.

===H2>
Notes on Triggers
===/H2>

   ISQL supports the use of triggers. Triggers are database operations that are 
   automatically performed when a specified database event  occurs. They are
   identified by name and specify a particular operation that is to be 
   performed on a specific table in a database when a certain event occurs.

   Triggers are set using the CREATE TRIGGER command and removed using the 
   DROP TRIGGER command.  There is no special BREW API support for them as
   none is needed.

   Triggers in ISQL behave according to the following rules:

   - A trigger created with the CREATE TRIGGER command is written to the 
     database file. It remains active until it is dropped using DROP TRIGGER. 
     Triggers are automatically loaded when the database file is opened and 
     thus outlive the ISQL instances used to create them.

  - When an application creates a trigger through ISQL_Exec(), other instances 
    of ISQL in the same or other applications become aware of the trigger the 
    next time they perform a read or write of the database. That is, when one 
    instance of ISQL creates a trigger, other instances of ISQL recognize the 
    database schema change when they next access it and automtically "learn" 
    about the new trigger. This all happens under the hood.

  - Triggers fire only once, and only in the context of the ISQL instance that 
    performed the trigger action. If, for example, application A adds a trigger, 
    then application B causes that trigger to fire, the trigger action is 
    performed in the context of application B only.

  - Temporary triggers created with CREATE TEMP TRIGGER behave just like regular 
    triggers, except that they are dropped when the ISQL instance that created 
    them is released. They are not written to the database and are dropped when 
    the database connection is closed.

  - As soon as a temporary trigger has been created from an ISQL instance, other 
    ISQL instances accessing the same database file automagically "learn" about 
    the temporary trigger and can cause it to fire. Once the ISQL instance that 
    created the temporary trigger is released, the trigger is immediately 
    dropped from other ISQL instances as well.
*
   Note that triggers (both temporary and not) fire as a result of an UPDATE, 
   INSERT or DELETE command. These commands can only be exectued when the ISQL 
   instance has write access to the database. If there's another instance of 
   ISQL reading the database at the time, the command fails and the trigger 
   doesn't fire. So there's no need to worry about readers when exectuing a 
   command that causes a trigger to fire.

===H2>
Required header files
===/H2>
===pre>
AEESQL.h
===/pre>

See Also:
   ISQLMgr
   ISQLStmt
   ISQLFunc

================================================================================

ISQL_AddRef() 

Description: 
   This function increments the reference count of the ISQL Interface object.  
   This allows the object to be shared by multiple callers. The object is freed 
   when the reference count reaches 0 (zero).

Prototype:
   uint32 ISQL_AddRef(ISQL *pISQL);

Parameters:
===pre>
   pISQL: Pointer to the ISQL Interface object
===/pre> 

Return Value:
   Incremented reference count for the object. 

Comments: 
   A valid object returns a positive reference count.

See Also:
   ISQL_Release() 

================================================================================

ISQL_Exec() 

Description: 
   This function compiles and executes the specified SQL text. Multiple 
   statements may be passed in using a semi-colon (";") as a statement 
   separator. Any query results are made available through the callback 
   function.

Prototype:
   uint32 ISQL_Exec(ISQL *piSQL, 
                    const char *pSQL, 
                    SQLExecCallBack pCallback,
                    void* pUserData, 
                    char **ppErrMsg);

Parameters:
===pre>
    piSQL:     Pointer to the ISQL Interface object
    pSQL :     Pointer to the SQL text
    pCallback: Pointer to callback function that receives query
               results. May be NULL if query results are not needed
    pUserData: Pointer that is passed in to the callback function
    ppErrMsg : Pointer to a pointer to a character that will point to a
               dynamically allocated string if an error occurs, which
               must be freed by the user.
===/pre>

Return Value:
    On success, SUCCESS is returned. Otherwise an error code is returned and
    ppErrMsg, if non-NULL, will point to a string describing the error.

Callback Prototype:
   int (*SQLExecCallBack)(void* pUserData, 
                          int nCols, 
                           char** ppColValues, 
                           char** ppColNames);

Callback Parameters:
===pre>
    pUserData:   Userdata pointer that was passed to ISQl_Exec
    nCols:       Number of columns of values and names
    ppColValues: Array of string pointers to results. First column is at 
                 ppColValues[0], 2nd is at ppColValues[1], etc.
    ppColNames:   Array of string pointers to column names. First column 
                  is at ppColNames[0], 2nd is at ppColNames[1], etc.
===/pre>

Callback Return Value:
    The callback should normally return 0. If the callback returns a non-zero
    value then the query is aborted, all subsequent SQL statements are skipped 
    and ISQL_Exec() function returns SQLITE_ABORT.
   
Comments: 
   None

Side Effects: 
   None

See Also:
    ISQL_Prepare() 

================================================================================

ISQL_GetError() 

Description: 
   This function returns the error code for the most recent failed  ISQL call. 

Prototype:
    uint32 ISQL_GetError(ISQL *piSQL);
    
Parameters:
===pre>
    pISQL: Pointer to the ISQL Interface object
===/pre>

Return Value: 
   ISQL error code

================================================================================

ISQL_GetRowsChanged() 

Description: 
   This function returns the number of database rows that were changed (or 
   inserted or deleted) by the most recently completed  INSERT, UPDATE, or 
   DELETE statement.

Prototype:
   int ISQL_GetRowsChanged(ISQL *piSQL);
    
Parameters:
===pre>
   pISQL : Pointer to the ISQL Interface object
===/pre>

Return Value: 
   Number of changed rows

================================================================================

ISQL_GetTotalChanged() 

Description: 
   This function returns the total number of database rows that have been 
   modified, inserted, or deleted since the database connection was created.

Prototype:
   int ISQL_GetTotalChanged(ISQL *piSQL);
    
Parameters:
===pre>
   pISQL: Pointer to the ISQL Interface object
===/pre>

Return Value: 
   Number of changed rows

================================================================================

ISQL_GetLastRowId() 

Description: 
   This function returns the integer key of most recent insert.  This  allows 
   applications to use the same ids the underlying SQL  implementation generates, 
   rather than generating their own ids.

   Each entry in an ISQL table has a unique integer key.  (The key is the value 
   of the INTEGER PRIMARY KEY column if there is such a column, otherwise the 
   key is generated at random.  The unique key is always available as the ROWID, 
   OID, or _ROWID_ column.)  The following routine returns the integer key of the 
   most recent insert in the database.

   This function is similar to the mysql_insert_id() function from MySQL.

Prototype:
   int64 ISQL_GetLastRowId(ISQL *piSQL);
    
Parameters:
===pre>
   pISQL: Pointer to the ISQL Interface object
===/pre>

Return Value: 
   Integer key of the most recent insert

================================================================================

ISQL_GetErrorMsgSize() 

Description: 
   This function returns the size of the error message generated by the most 
   recently failed ISQL call.  Call this function to determine how big a
   buffer to allocate for the error message itself.

Prototype:
   int ISQL_GetErrorMsgSize(ISQL *piSQL);
    
Parameters:
===pre>
   pISQL: Pointer to the ISQL Interface object
===/pre>

Return Value: 
   Size of the last error message generated by the last SQL operation

See Also:
   ISQL_GetErrorMsg()

================================================================================

ISQL_GetErrorMsg() 

Description: 
   This function copies the last error message generated by ISQL into the
   supplied buffer.  The buffer passed in must be non-null.  Typcially, an
   application would first call ISQL_GetErrorMsgSize() to determine how big a
   buffer is needed, allocate memory for that buffer, then pass it to
   ISQL_GetErrorMsg() to retrieve the last error message.

Prototype:
   int ISQL_GetErrorMsg(ISQL *piSQL, char *msg, int size);
    
Parameters:
===pre>
   pISQL: Pointer to the ISQL Interface object
   msg:   buffer (non-null) to copy the error message into
   size:  size of the msg buffer
===/pre>

Return Value: 
   Any error that occurred copying the last error message

See Also:
   ISQL_GetErrorMsgSize()

================================================================================
      
ISQL_Prepare() 

Description: 
   This function is used to compile a SQL statement so it can be  executed using 
   ISQLSTMT_Step(). This function returns a pointer to an ISQLStmt object. There 
   are 2 forms of this function depending on whether the SQL source is in 8-bit 
   or 16-bit format. If there are multiple statements, a pointer to the next 
   statement to compile is returned.

Prototype:
    int ISQL_Prepare(ISQL *piSQL, 
                     const char *pSQL, 
                     int len, 
                     ISQLStmt **ppStmt, 
                     const char **ppsTail);
    
    int ISQL_WPrepare(ISQL *piSQL, 
                      const AECHAR *pSQL, 
                      int len, 
                      ISQLStmt **ppStmt, 
                      const AECHAR **ppsTail);

Parameters:
===pre>
    pISQL:   Pointer to the ISQL Interface object
    pSQL:    Pointer to SQL text as 8-bit char or 16-bit AECHAR
    len:     Number of characters in SQL text
    ppStmt:  Pointer to returned ISQLStmt object. This must be
             release when it is no longer needed.
    ppsTail: If there are multiple statments, this will point
             to the next statement.
===/pre>

Return Value: 
    On success, SUCCESS is returned. Otherwise an error code is returned.

See Also:
    ISQL_Exec()
    ISQLSTMT_Step()
    ISQLSTMT_Reset()

================================================================================

ISQL_QueryInterface()

Description:
   This function queries an interface for an alterrnate interface that may 
   provide extended functionality or a proprietary extension.  If successful, 
   the function will return a pointer to the requested interface in the memory 
   location pointed to by the 'd' parameter.  The requested interface is 
   identified by a globally unique interface ID passed to the function in the 
   'c' parameter.

Prototype:
   int ISQL_QueryInterface(ISQL *p, AEECLSID c, void **d);

Parameters:
===pre>
   p:  Pointer to the ISQL interface object.
   c:  A globally unique id that identifies the interface to be queried.
   d:  Pointer to a pointer returned by the function to reference the
       interface requested by the 'c' parameter.  If the queried item is not
       available, this pointer is returned as NULL.
===/pre> 

Return Value:
===pre>
   SUCCESS          - The requested interface was found and has been returned
                      in the 'd' parameter.
   ECLASSNOTSUPPORT - The requested interface is not supported.
===/pre> 

Comments:
   The pointer passed in the 'd' parameter is a double-indirection -- i.e. it 
   is a pointer to a pointer.  Applications should use care in ensuring that the 
   proper level of indirection is used in calling the function.

Side Effects:
   Upon successful retrieval of an interface pointer, that interface is returned 
   with its reference count incremented.  The caller is responsible for 
   releasing this object at some point in the future.

See Also:
   IQI_QueryInterface()

================================================================================

ISQL_Release() 

Description: 
   This function decrements the reference count of the ISQL Interface object. 
   The object is freed from memory and is no longer valid once the reference 
   count reaches 0 (zero). 

Prototype:
   uint32 ISQL_Release(ISQL *pISQL);
    
Parameters:
===pre>
   pISQL: Pointer to the ISQL Interface object
===/pre> 

Return Value:
===pre> 
   The decremented reference count for the object.
   0 (zero) if the object has been freed and is no longer valid.
===/pre> 

See Also:
   ISQL_AddRef() 

================================================================================
  DATA STRUCTURE DOCUMENTATION
================================================================================
ISQLStmt Interface and Class IDs

Description:
   Each class and interface is identified by a unique ID.  These IDs are used to 
   distinguish between various types of objects.

   ISQLStmt supports both an interface and the creation of class object 
   instances. The ISQLStmt interface or a new instance of the ISQLStmt class 
   object may be created by calling ISHELL_CreateInstance() and passing in the 
   appropriate class id.

   ISQLStmt recognizes the following set of class and interface IDs.

Definition:

Identifier           Definition
----------           -----------------------------------------------------------
AEEIID_SQLSTMT:      Identifies the ISQLStmt interface
AEECLSID_SQLSTMT:    Identifies the ISQLStmt implementation in the SQL extension

Comments:
   None

See Also:
   ISHELL_CreateInstance()

================================================================================

AEESQLDataType

Description:
   ISQL supports manifest typing - any column in the database can hold any type
   of value.  The following constants identify the data types that ISQL 
   supports:
   
Definition:
===pre>
   Definition              Description
   ----------------------  -------------------------------------------------------
   AEESQL_DT_TEXT          Single-byte character-based text
   AEESQL_DT_WTEXT         Wide character-based text
   AEESQL_DT_INT           An integer value, 32 bits or less
   AEESQL_DT_DOUBLE        A float/double value
   AEESQL_DT_BLOB          A binary blob of data
   AEESQL_DT_NULL          NULL, no data at all
   AEESQL_DT_INT64         int64 data
===/pre>

Members:
   None
   
Comments:
   None

See Also:
   ISQLSTMT_GetColumn()
   ISQLSTMT_BindData()

================================================================================

AEESQLColDataType

Description:
   ISQL supports manifest typing - any column in the database may contain data
   of any of the support types - but each column in a table is set up with a
   preferred data types.  The preferred data type of table columns is 
   identified with one of the following constants:
   
Definition:
===pre>
   Definition              Description
   ----------------------  -------------------------------------------------------
   AEESQL_COL_DT_INTEGER   Integer data, 64 bit or less
   AEESQL_COL_DT_FLOAT     Float/double data
   AEESQL_COL_DT_TEXT      Text data, either single or double byte
   AEESQL_COL_DT_BLOB      Binary data
   AEESQL_COL_DT_NULL      NULL, no data at all
===/pre>

Members:
   None
   
Comments:
   None

See Also:
   ISQLSTMT_ColumnDataType()

================================================================================

SQLFreeCallBack

Description:
   This prototype defines the callback passed to ISQLStmt to free bind data.

Definition:
   typedef void (*SQLFreeCallBack)(void* pData);

Members:
===pre>
   pData:  A pointer to the data that needs to be freed
===/pre>

Comments:
   None

See Also:
   ISQLSTMT_BindBlob()
   ISQLSTMT_BindText()
   ISQLSTMT_BindWText()

================================================================================
 INTERFACE DOCUMENTATION
================================================================================

ISQLStmt Interface

Description: 

   The ISQLStmt interface functions allow you to perform the following on 
   compiled SQL statements: execute a statement, get column results in a 
   specified type, get information about result columns, and bind data to 
   queries.

   Statement execution is done with ISQLSTMT_Step. ISQLSTMT_Reset can be used 
   after stepping to reset the statement back to its state after ISQL_Prepare() 
   so it can be executed again from the beginning.

   After running ISQLSTMT_Step(), use any of the following to get column 
   results: ISQLSTMT_GetColumnBlob(),  ISQLSTMT_GetColumnInt(),
   ISQLSTMT_GetColumnInt64(), ISQLSTMT_GetColumnDouble(), 
   ISQLSTMT_GetColumnText(), or ISQLSTMT_GetColumnWText(). If the data is not 
   currently in the type specified, it will be automatically converted.

   Use ISQLSTMT_ColumnCount() to get the number of columns. Use 
   ISQLSTMT_ColumnName() to get the name of a column. Use 
   ISQLSTMT_WColumnName() to get the name as a wide string. Use 
   ISQLSTMT_ColumnDataType() to get the type of data stored in a column. Use 
   ISQLSTMT_ColumnDeclType() or ISQLSTMT_WColumnDeclType() to get the declared 
   type, if any, of a column. Use ISQLSTMT_ColumnSize() to get the size of 
   data in a column. This is most useful for blob or text data.

   Binding allows you to add and update data in binary or numeric format 
   without having to first convert to text. The bound data replaces wildcards 
   that are in the SQL text. Wildcards include "?" or "?nnn" where "nnn" is 
   an integer or ":aaa" or "$aaa" where "aaa" is an alphanumeric identifier. 
   Each wildcard has an associated number which is one more than the number 
   of prior wildcards in the same SQL statement or is the number provided 
   with a "?nnn" wildcard. It is allowed for the same wildcard to occur more 
   than once in the same SQL statement, in which case all instance of that 
   wildcard will be filled in with the same value. Unbound wildcards have a 
   value of NULL. Use the following to bind data to a wildcard: 
   ISQLSTMT_BindBlob(), ISQLSTMT_BindInt(), ISQLSTMT_BindInt64(), 
   ISQLSTMT_BindDouble(), ISQLSTMT_BindText(), ISQLSTMT_BindWText(), or 
   ISQLSTMT_BindNull(). ISQLSTMT_BindCount() indicates how many unique 
   wildcards there are. ISQLSTMT_BindIndex() returns the associated number 
   for a wildcard that is specified by name. Note that the name must include 
   the preceding token, for example ":address" or "$phone". ISQLSTMT_BindName() 
   returns the name of a wildcard given it's number.

===H2>
Required header files
===/H2>
===pre>
AEESQL.h
===/pre>

See Also:
   ISQL

================================================================================

ISQLSTMT_AddRef() 

Description: 
   This function increments the reference count of the ISQLStmt Interface object.  
   This allows the object to be shared by multiple callers. The object is freed 
   when the reference count reaches 0 (zero).

Prototype:
   uint32 ISQLSTMT_AddRef(ISQLStmt *piStmt);

Parameters:
===pre>
   piStmt: Pointer to the ISQLStmt Interface object
===/pre> 

Return Value:
   Incremented reference count for the object. 

Comments: 
   A valid object returns a positive reference count.

See Also:
   ISQLSTMT_Release() 

================================================================================

ISQLSTMT_BindBlob()

Description: 
   These binding functions allow you to add and update data in binary or 
   numeric format without having to first convert to text. The bound data 
   replaces wildcards that are in the SQL text. Wildcards include "?" or "?nnn"
   where "nnn" is an integer or ":aaa" or "$aaa" where "aaa" is an alphanumeric 
   identifier. Each wildcard has an associated number which is one more than the 
   number of prior wildcards in the same SQL statement or is the number provided 
   with a "?nnn" wildcard.  If named wildcards are used, use the function 
   ISQLSTMT_BindIndex() to get the number from the name.

   It is allowed for the same wildcard to occur more than once in the same SQL 
   statement, in which case all instances of that wildcard will be filled in 
   with the same value. Unbound wildcards have a value of NULL.

Prototype:
   int ISQLSTMT_BindBlob(ISQLStmt *piStmt, 
                         int wildcard, 
                         void *pBlob,
                         int nSize, 
                         SQLFreeCallBack pCB);

Parameters:
===pre>
   piStmt:   Pointer to the ISQLStmt Interface object
   wildcard: numeric value of wildcard to replace
   pBlob:    pointer to any type of data
   nSize:    size (in bytes) of data
   pCB:      pointer to callback function to free data if it needs to be freed
             (SQLFreeCallBack)(0) or NULL if the data doesn't need to be freed
             (SQLFreeCallBack)(-1) if ISQL should make a copy of the data
===/pre> 

Return Value:
   On success, SUCCESS is returned. Otherwise an error code is returned.

================================================================================

ISQLSTMT_BindInt()

Prototype:
   int ISQLSTMT_BindInt(ISQLStmt *piStmt, int wildcard, int value);

Parameters:
===pre>
   piStmt:   Pointer to the ISQLStmt Interface object
   wildcard: numeric value of wildcard to replace
   value:    integer value
===/pre>

Return Value:
   On success, SUCCESS is returned. Otherwise an error code is returned.

================================================================================

ISQLSTMT_BindInt64()

Prototype:
   int ISQLSTMT_BindInt64(ISQLStmt *piStmt, int wildcard, int64 value);

Parameters:
===pre>
   piStmt:   Pointer to the ISQLStmt Interface object
   wildcard: numeric value of wildcard to replace
   value:    64-bit integer value
===/pre>

Return Value:
   On success, SUCCESS is returned. Otherwise an error code is returned.

================================================================================

ISQLSTMT_BindDouble()

Prototype:
   int ISQLSTMT_BindDouble(ISQLStmt *piStmt, int wildcard, double value);

Parameters:
===pre>
   piStmt:   Pointer to the ISQLStmt Interface object
   wildcard: numeric value of wildcard to replace
   value:    floating point value
===/pre>

Return Value:
   On success, SUCCESS is returned. Otherwise an error code is returned.

================================================================================

ISQLSTMT_BindText()

Prototype:
   int ISQLSTMT_BindText(ISQLStmt *piStmt, 
                         int wildcard, 
                         char *pText, 
                         int nSize, 
                         SQLFreeCallBack pCB);

Parameters:
===pre>
   piStmt:   Pointer to the ISQLStmt Interface object
   wildcard: numeric value of wildcard to replace
   pText:    pointer to text
   nSize:    size (in bytes) of text
   pCB:      pointer to callback function to free data if it needs to be freed
             (SQLFreeCallBack)(0) or NULL if the data doesn't need to be freed
             (SQLFreeCallBack)(-1) if ISQL should make a copy of the data
===/pre>

Return Value:
   On success, SUCCESS is returned. Otherwise an error code is returned.

================================================================================

ISQLSTMT_BindWText()

Prototype:
   int ISQLSTMT_BindWText(ISQLStmt *piStmt, 
                          int wildcard, 
                          AECHAR *pWText,
                          int nSize, 
                          SQLFreeCallBack pCB);

Parameters:
===pre>
   piStmt:   Pointer to the ISQLStmt Interface object
   wildcard: numeric value of wildcard to replace
   pWText:   pointer to wide-character text
   nSize:    size (in bytes) of text
   pCB:      pointer to callback function to free data if it needs to be freed
             (SQLFreeCallBack)(0) or NULL if the data doesn't need to be freed
             (SQLFreeCallBack)(-1) if ISQL should make a copy of the data
===/pre>

Return Value:
   On success, SUCCESS is returned. Otherwise an error code is returned.

================================================================================

ISQLSTMT_BindNull()

Prototype:
   int ISQLSTMT_BindNull(ISQLStmt *piStmt, int wildcard);

Parameters:
===pre>
   piStmt:   Pointer to the ISQLStmt Interface object
   wildcard: numeric value of wildcard to replace
===/pre>

Return Value:
   On success, SUCCESS is returned. Otherwise an error code is returned.

See Also:
   ISQLSTMT_BindCount()
   ISQLSTMT_BindIndex()
   ISQLSTMT_BindName()

================================================================================

ISQLSTMT_BindCount()

Description: 
   Returns the number of the wildcard with the highest number. Normally this is 
   the number of wildcards but it may differ if the "?nnn" format is used and 
   numbers are skipped.

Prototype:
   int ISQLSTMT_BindCount(ISQLStmt *piStmt);

Parameters:
===pre>
   piStmt: Pointer to the ISQLStmt Interface object
===/pre>

Return Value:
   Number of wildcards or number of wildcard with highest number.

See Also:
   ISQLSTMT_BindIndex()
   ISQLSTMT_BindName()

================================================================================

ISQLSTMT_BindIndex()

Description: 
   Given the name of a bind wildcard, returns the associated number.

Prototype:
   int ISQLSTMT_BindIndex(ISQLStmt *piStmt, const char *pName);

Parameters:
===pre>
   piStmt: Pointer to the ISQLStmt Interface object
   pName:  Wildcard label
===/pre>

Return Value:
   Number of wildcard

See Also:
   ISQLSTMT_BindCount()
   ISQLSTMT_BindName()

================================================================================

ISQLSTMT_BindName()

Description: 
   Given the number of a bind wildcard, returns the associated name if there is 
   one.

Prototype:
   const char* ISQLSTMT_BindName(ISQLStmt *piStmt, int index);

Parameters:
===pre>
   piStmt: Pointer to the ISQLStmt Interface object
   index:  Wildcard number
===/pre>

Return Value:
   Name of wildcard

See Also:
   ISQLSTMT_BindCount()
   ISQLSTMT_BindIndex()

================================================================================

ISQLSTMT_ColumnCount()

Description: 
   Returns the number of columns in a query result.

Prototype:
   int ISQLSTMT_ColumnCount(ISQLStmt *piStmt);

Parameters:
===pre>
   piStmt: Pointer to the ISQLStmt Interface object
===/pre>

Return Value:
   Number of columns

================================================================================

ISQLSTMT_ColumnDataType()

Description: 
   Returns the type of data stored in the specified column of a query result.

Prototype:
   AEESQLColDataType ISQLSTMT_ColumnDataType(ISQLStmt *piStmt, int column);

Parameters:
===pre>
   piStmt: Pointer to the ISQLStmt Interface object
   column: Index of 0-based column
===/pre>

Return Value:
===pre>
   Type of data in column. One of:
      AEESQL_COL_DT_INTEGER,
      AEESQL_COL_DT_FLOAT,
      AEESQL_COL_DT_TEXT,
      AEESQL_COL_DT_BLOB,
      AEESQL_COL_DT_NULL
===/pre>
    
See Also:
   ISQLSTMT_ColumnDeclType()
   ISQLSTMT_ColumnName()

================================================================================

ISQLSTMT_ColumnDeclType()
ISQLSTMT_WColumnDeclType()

Description: 
   Returns the date type, if any, that may be optionally declared in the CREATE 
   TABLE statement for a column.

Prototype:
   const char * ISQLSTMT_ColumnDeclType(ISQLStmt *piStmt, int column);
   AECHAR * ISQLSTMT_WColumnDeclType(ISQLStmt *piStmt, int column);

Parameters:
===pre>
   piStmt: Pointer to the ISQLStmt Interface object
   column: Index of 0-based column
===/pre>

Return Value:
   Type of data declared for the column, if any as text pointer or wide
   character pointer.
    
See Also:
   ISQLSTMT_ColumnDataType()
   ISQLSTMT_ColumnName()

================================================================================

ISQLSTMT_ColumnName()
ISQLSTMT_WColumnName()

Description: 
   Returns the declared name of a column.

Prototype:
   const char * ISQLSTMT_ColumnName(ISQLStmt *piStmt, int column);
   AECHAR * ISQLSTMT_WColumnName(ISQLStmt *piStmt, int column);

Parameters:
===pre>
   piStmt: Pointer to the ISQLStmt Interface object
   column: Index of 0-based column
===/pre>

Return Value:
   Name of the column as text pointer or wide-character pointer.
    
See Also:
   ISQLSTMT_ColumnDataType()
   ISQLSTMT_ColumnDeclType()

================================================================================

ISQLSTMT_ColumnSize()

Description: 
   Returns the data size for a column.

Prototype:
   int ISQLSTMT_ColumnSize(ISQLStmt *piStmt, int column);
    
Parameters:
===pre>
   piStmt: Pointer to the ISQLStmt Interface object
   column: Index of 0-based column
===/pre>

Return Value:
   Size, in bytes, of data for this column.
    
See Also:
   ISQLSTMT_ColumnDataType()
   ISQLSTMT_ColumnName()

================================================================================

ISQLSTMT_GetColumn()

Description: 
   This routine returns the data in the specified column of the current row of
   an ISQLStmt object.  The column index is zero based, with the left-most 
   column having an index of 0.  If the SQLStmt is not initialized, not 
   currently pointing to a valid row or if the the column index is out of range, 
   the result is undefined.

   The caller specifies the type of the column data in this call as well as a 
   pointer that will either point to or contain the column data on return.  The
   caller is also repsonsible for passing in the size of the pointer.  Note that
   this routine will attempt to convert the column data into the specified type.

   If a column type of AEESQL_COL_DT_TEXT or AEESQL_COL_DT_BLOB is specified
   by the caller, the pData parameter will point to the column data on return.
   The caller can then call ISQLSTMT_ColumnSize() to determine the size of
   the data that was returned.
   
   If a column type of AEESQL_COL_DT_INTEGER or AEESQL_COL_DT_FLOAT is
   specified, the pData paramter must point to a buffer large enough for the 
   result.  The size parameter must specify the size of the buffer as well.
   On return, the buffer pointed to by pData will contain a copy of the column 
   data.

   Applications may call this routine with the appropriate parameters set or
   use one of the helper static inlines defined:

===pre>
   ISQLSTMT_GetColumnBlob()
   ISQLSTMT_GetColumnInt()
   ISQLSTMT_GetColumnInt64()
   ISQLSTMT_GetColumnDouble()
   ISQLSTMT_GetColumnText()
   ISQLSTMT_GetColumnWText()
===/pre>

   For more information on how this routine or the shortcuts above are used,
   refer to the ISQLStmt interface description.

Prototype:
   int ISQLSTMT_GetColumn(ISQLStmt *piStmt, 
                          AEESQLDataType type, 
                          int column, 
                          void *pData, 
                          int size);
    
Parameters:
===pre>
   piStmt: Pointer to the ISQLStmt Interface object
   type:   AEESQLDataType value indicating the column's data type
   column: Index of 0-based column
   pData:  On return, either contains or points to column data
   size:   On input, size of pData
===/pre>

Return Value:
   SUCCESS if the column data was retrieve
   EBADPARM if a bogus column or data type was requested
   any other error that may have occurred
    
See Also:
   AEESQLDataType
   ISQLSTMT_ColumnSize()
   ISQLStmt

================================================================================

ISQLSTMT_QueryInterface()

Description:
   This function queries an interface for an alterrnate interface that may 
   provide extended functionality or a proprietary extension.  If successful, 
   the function will return a pointer to the requested interface in the memory 
   location pointed to by the 'd' parameter.  The requested interface is 
   identified by a globally unique interface ID passed to the function in the 
   'c' parameter.

Prototype:
   int ISQLSTMT_QueryInterface(ISQLStmt *p, AEECLSID c, void **d);

Parameters:
===pre>
   p:  Pointer to the ISQLStmt interface object.
   c:  A globally unique id that identifies the interface to be queried.
   d:  Pointer to a pointer returned by the function to reference the
       interface requested by the 'c' parameter.  If the queried item is not
       available, this pointer is returned as NULL.
===/pre> 

Return Value:
===pre>
   SUCCESS          - The requested interface was found and has been returned
                      in the 'd' parameter.
   ECLASSNOTSUPPORT - The requested interface is not supported.
===/pre> 

Comments:
   The pointer passed in the 'd' parameter is a double-indirection -- i.e. it 
   is a pointer to a pointer.  Applications should use care in ensuring that the 
   proper level of indirection is used in calling the function.

Side Effects:
   Upon successful retrieval of an interface pointer, that interface is returned 
   with its reference count incremented.  The caller is responsible for 
   releasing this object at some point in the future.

See Also:
   IQI_QueryInterface()

================================================================================
 
ISQLSTMT_Release() 

Description: 
   This function decrements the reference count of the ISQLStmt Interface object. 
   The object is freed from memory and is no longer valid once the reference 
   count reaches 0 (zero). 

Prototype:
   uint32 ISQLSTMT_Release(ISQLStmt *piStmt);
    
Parameters:
===pre>
   piStmt: Pointer to the ISQLStmt Interface object
===/pre> 

Return Value:
===pre> 
   The decremented reference count for the object.
   0 (zero) if the object has been freed and is no longer valid.
===/pre> 

See Also:
   ISQLSTMT_AddRef() 

================================================================================

ISQLSTMT_Reset() 

Description: 
   This function resets a prepared ISQL statement back to its initial state, as 
   if it had just been prepared using ISQL_Prepare().

Prototype:
   int ISQLSTMT_Reset(ISQLStmt *piStmt);
    
Parameters:
===pre>
   piStmt: Pointer to the ISQLStmt Interface object
===/pre>

Return Value: 
   On success, SUCCESS is returned. Otherwise an error code is returned.

See Also:
   ISQL_Prepare()
   ISQLSTMT_Step() 

================================================================================

ISQLSTMT_Step() 

Description: 
   This function steps through a compiled statement that has been prepared by
   ISQL_Prepare(). This can be executed as many times as there are statements
   compiled in the ISQLStmt object.

Prototype:
   int ISQLSTMT_Step(ISQLStmt *piStmt);
    
Parameters:
===pre>
   piStmt: Pointer to the ISQLStmt Interface object
===/pre>

Return Value: 
   One of: SQL_BUSY, SQL_DONE, SQL_ROW, SQL_ERROR, or SQL_MISUSE.

See Also:
   ISQL_Prepare()
   ISQLSTMT_Reset() 

================================================================================
  DATA STRUCTURE DOCUMENTATION
================================================================================
ISQLUtil Interface and Class IDs

Description:
   Each class and interface is identified by a unique ID.  These IDs are used to 
   distinguish between various types of objects.

   ISQLUtil supports both an interface and the creation of class object 
   instances. The ISQLUtil interface or a new instance of the ISQLUtil class 
   object may be created by calling ISHELL_CreateInstance() and passing in the 
   appropriate class id.

   ISQLUtil recognizes the following set of class and interface IDs.

Definition:

Identifier           Definition
----------           -----------------------------------------------------------
AEEIID_SQLUTIL:      Identifies the ISQLUtil interface
AEECLSID_SQLUTIL:    Identifies the ISQLUtil implementation in the SQL extension

Comments:
   None

See Also:
   ISHELL_CreateInstance()

================================================================================
 INTERFACE DOCUMENTATION
================================================================================

ISQLUtil Interface

Description: 
   The ISQLUtil interface contains utility functions which may be useful for 
   developing code with the ISQL interface.

===H2>
Required header files
===/H2>
===pre>
AEESQL.h
===/pre>

See Also:
   None

================================================================================

ISQLUTIL_AddRef() 

Description: 
   This function increments the reference count of the ISQLUtil Interface 
   object.  This allows the object to be shared by multiple callers. The object 
   is freed when the reference count reaches 0 (zero).

Prototype:
   uint32 ISQLUTIL_AddRef(ISQLUtil *pISQLUtil);

Parameters:
===pre>
   pISQLUtil: Pointer to the ISQLUtil Interface object
===/pre> 

Return Value:
   Incremented reference count for the object. 

Comments: 
   A valid object returns a positive reference count.

See Also:
   ISQLUTIL_Release() 

================================================================================

ISQLUTIL_QueryInterface()

Description:
   This function queries an interface for an alterrnate interface that may 
   provide extended functionality or a proprietary extension.  If successful, 
   the function will return a pointer to the requested interface in the memory 
   location pointed to by the 'd' parameter.  The requested interface is 
   identified by a globally unique interface ID passed to the function in the 
   'c' parameter.

Prototype:
   int ISQLUTIL_QueryInterface(ISQLUtil *p, AEECLSID c, void **d);

Parameters:
===pre>
   p:  Pointer to the ISQLUtil interface object.
   c:  A globally unique id that identifies the interface to be queried.
   d:  Pointer to a pointer returned by the function to reference the
       interface requested by the 'c' parameter.  If the queried item is not
       available, this pointer is returned as NULL.
===/pre> 

Return Value:
===pre>
   SUCCESS          - The requested interface was found and has been returned
                      in the 'd' parameter.
   ECLASSNOTSUPPORT - The requested interface is not supported.
===/pre> 

Comments:
   The pointer passed in the 'd' parameter is a double-indirection -- i.e. it 
   is a pointer to a pointer.  Applications should use care in ensuring that the 
   proper level of indirection is used in calling the function.

Side Effects:
   Upon successful retrieval of an interface pointer, that interface is returned 
   with its reference count incremented.  The caller is responsible for 
   releasing this object at some point in the future.

See Also:
   IQI_QueryInterface()

================================================================================

ISQLUTIL_Release() 

Description: 
   This function decrements the reference count of the ISQLUtil Interface object. 
   The object is freed from memory and is no longer valid once the reference 
   count reaches 0 (zero). 

Prototype:
   uint32 ISQLUTIL_Release(ISQLUtil *pISQLUtil);
    
Parameters:
===pre>
   pISQLUtil: Pointer to the ISQLUtil Interface object
===/pre> 

Return Value:
===pre> 
   The decremented reference count for the object.
   0 (zero) if the object has been freed and is no longer valid.
===/pre> 

See Also:
   ISQLUTIL_AddRef() 

================================================================================

ISQLUTIL_vxnprintf()

Description:
   This function writes formatted output to a character buffer.  The format is 
   specified using a format string, which follows the format rules of the
   standard c printf function.  The format consists of a combination of regular 
   characters, which are passed along unchanged, and conversion specifiers, 
   which may take an argument and produced format output.

   A conversion specifier begins with a required %, followed by an optional 
   flag, followed by an option field width, followed by an optional precision,
   followed by a required conversion specifier.  Please refer to the standard C 
   printf documentation for more details.

Flags:
===pre>
   #, 0, -, (space), and + flags work the same as the c printf function.
   Please see standard C printf documentation for the function of these flags. 
   Combining multiple flags results in undefined behavior.
===/pre> 

Conversion specifiers:
===pre>
   d, i, o, u, x, X, f, e, E, g, G, c, s, n, % work the same as the c printf
   function.  Please see standard c printf documentation for more details.
===/pre> 

Two extended conversion specifiers are supported:
===pre>
   q: This is used for SQL escaping. It accepts a string as an argument and 
      doubles the ' characters in the string.
   Q: Similar to q, the ' characters are doubled.  In addition, the entire 
      string is enclosed in ' characters.
===/pre> 

Prototype:
   ISQLUTIL_vxnprintf(ISQLUtil *piUtil, 
                      int useInternal, 
                      char *buf, 
                      uint32 f, 
                      const char *format, 
                      va_list list)';

Parameters:
===pre>
    piUtil:       Pointer to the ISQLUtil Interface object
    useInternal:  1 to perform internal % conversions, 0 otherwise
    buf:          Storage location for output
    f:            Size of buf
    format:       Format control string
    list:         List of arguments to the format control string
===/pre> 

Return Value:
   The length of the formatted string

Comments:
   None

==============================================================================*/


#endif   // __AEEISQL_H__
